#ifndef DRAWOBJ_HG_INC
#define DRAWOBJ_HG_INC

class ObjectBaseClass
{
public:
   virtual void Draw(SmartWin::Canvas &aCanvas, bool aMoving = false) = 0;
   virtual ~ObjectBaseClass() {}
   virtual void SetEnd(SmartWin::Point aEnd) = 0;
   virtual void Write(std::ostream &os) = 0;
   virtual bool Read(std::istream &is) = 0;
};

class LineClass : public ObjectBaseClass
{
public:
   LineClass() {}
   LineClass(SmartWin::Point aStart, COLORREF aColor) : Start(aStart), End(aStart), Color(aColor)
   {}
   void Draw(SmartWin::Canvas &aCanvas, bool aMoving)
   {
      SmartWin::HdcModeSetter Mode(aCanvas, aMoving ? R2_NOTXORPEN : R2_COPYPEN);
      SmartWin::Pen pen(aCanvas, Color);
      aCanvas.MoveTo(Start);
      aCanvas.LineTo(End);
   }
   void SetEnd(SmartWin::Point aEnd)
   {
      End = aEnd;
   }
   virtual void Write(std::ostream &os);
   virtual bool Read(std::istream &is);
private:
   SmartWin::Point Start, End;
   COLORREF Color;
};

class RectangleClass : public ObjectBaseClass
{
public:
   RectangleClass() {}
   RectangleClass(SmartWin::Point aStart, COLORREF aColor) :
      Start(aStart), End(aStart), Color(aColor)
   {}

   void SetEnd(SmartWin::Point aEnd)
   {
      End = aEnd;
   }
   void Draw(SmartWin::Canvas &aCanvas, bool aMoving = false);
private:
   SmartWin::Point Start, End;
   COLORREF Color;
   virtual void Write(std::ostream &os);
   virtual bool Read(std::istream &is);
};

inline std::ostream &operator << (std::ostream &os, const SmartWin::Point &aP)
{
   os << aP.x << " " << aP.y;
   return os;
}

inline std::istream &operator >> (std::istream &is, SmartWin::Point &aP)
{
   is >> aP.x;
   is >> aP.y;
   return is;
}

#endif
